
/******************************************************************************
 * @file     VADC.c
 * @brief    Queue Source converts channels 2, 4, 5, 1, 3, 6 and 7 in Group 0
 * 			 and results are presented in two FIFO queues.
 * @version  V1.0
 * @date     10. September 2012
 * @note
 * Copyright (C) 2012 Infineon Technologies AG. All rights reserved.
 ******************************************************************************
 * @par
 * Infineon Technologies AG (Infineon) is supplying this software for use with
 * Infineons microcontrollers.
 * This file can be freely distributed within development tools that are
 * supporting such microcontrollers.
 * @par
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *****************************************************************************/

#include "VADC.h"

//GLobal variables
uint32_t result1=0;
uint32_t result10[16],result11[16],result12[16],result13[16];

/*****************************************************************************/
//  VADC_Init()
//  Initializes VADC
//  Queue Source is converting continuously seven no-consecutive
//  channels (2,4,5,1,3,6,7) in G0.
//  It is configured as SW triggered in Channel 2.
//	Result are stored in two FIFO queues, one with three values (Ch. 2,4,5)
//  from Result Register 0 to 2 and other with four values (Ch. 1,3,6,7)
//  from register 3 to 7.
//
// 	Get_Result function is defined in order to get results in main.
/*****************************************************************************/

void VADC_Init()
{

//De-asserted reset for ADC
	 	SCU_RESET->PRCLR0  = SCU_RESET_PRCLR0_VADCRS_Msk ;


//GLOBAL VADC INITIALIZATION

	 	VADC->CLC          = (0 << VADC_CLC_DISR_Pos & VADC_CLC_DISR_Msk) |
							  // DISR = 0: Module Disable Request Bit
							  // 0B On request: enable the module clock
						     (0 << VADC_CLC_DISS_Pos & VADC_CLC_DISS_Msk) |
							  // DISS = 0: Module Disable Status Bit
							  // 0B Module clock is enabled
						     (0 << VADC_CLC_EDIS_Pos & VADC_CLC_EDIS_Msk);
							  // EDIS = 0: Sleep Mode Enable Control
							  // 0B Sleep mode request is enabled and functional

// wait for module clock enabled
	 	while(VADC->CLC!=0);


	 	VADC->GLOBCFG      = (3 << VADC_GLOBCFG_DIVA_Pos & VADC_GLOBCFG_DIVA_Msk) |
							  // DIVA = 3: Divider Factor for the Analog Internal Clock
							  //  fADCI = fADC / (DIVA + 1) = fADC / 4
						     (1 << VADC_GLOBCFG_DIVWC_Pos & VADC_GLOBCFG_DIVWC_Msk);
							  // DIVWC = 1: Write Control for Divider Parameters
							  //  Bitfields DIVA, DCMSB, DIVD can be written

	//QUEUE SOURCE

		VADC_G0->QCTRL0	   = (0 << VADC_G_QCTRL0_XTWC_Pos & VADC_G_QCTRL0_XTWC_Msk) |
							   //Write Control for Trigger Configuration. 0B Bitfields XTMODE and XTSEL can not be written
							 (0 << VADC_G_QCTRL0_GTWC_Pos & VADC_G_QCTRL0_GTWC_Msk);
							   //Write Control for Gate Configuration. 0B Bitfield GTSEL can not be written
		VADC_G0->QINR0 	   = ((0x2 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								// Request Channel Number
								//Defines the channel number to be converted
							 ((1<< VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								// Refill

		VADC_G0->QINR0 	   = ((0x4 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								// Request Channel Number
								//Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								// Refill

		VADC_G0->QINR0 	   = ((0x5 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								// Request Channel Number
								//Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								 // Refill

		VADC_G0->QINR0 	   = ((0x1 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
									// Request Channel Number
									//Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								// Refill

		VADC_G0->QINR0 	   = ((0x3 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								// Request Channel Number
								//Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								 // Refill

		VADC_G0->QINR0 	   = ((0x6 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								 // Request Channel Number
								 //Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								 //Refill

		VADC_G0->QINR0 	   = ((0x7 << VADC_G_QINR0_REQCHNR_Pos) & VADC_G_QINR0_REQCHNR_Msk) |
								// Request Channel Number
								//Defines the channel number to be converted
							 ((1 << VADC_G_QINR0_RF_Pos) & VADC_G_QINR0_RF_Msk);
								 // Refill

		VADC_G0->QMR0	   = ((1 << VADC_G_QMR0_ENGT_Pos) & VADC_G_QMR0_ENGT_Msk)|
								  //Conversion requests are issued if a valid conversion request is pending in the queue 0 register
								  //or in the backup register
							 ((1 << VADC_G_QMR0_TREV_Pos) & VADC_G_QMR0_TREV_Msk);

//CHANNEL CONFIGURATION

//G0
		VADC_G0->CHCTR[1]  = ((7 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								  //Result Register: 7
		VADC_G0->CHCTR[2]  = ((2 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 2
		VADC_G0->CHCTR[3]  = ((7 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 7
		VADC_G0->CHCTR[4]  = ((2 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 2
		VADC_G0->CHCTR[5]  = ((2 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 2
		VADC_G0->CHCTR[6]  = ((7 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 7
		VADC_G0->CHCTR[7]  = ((7 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
								 //Result Register: 7

//FIFO Result accumulation 1

	 	VADC_G0->RCR[0]	   = ((1 << VADC_G_RCR_FEN_Pos) & VADC_G_RCR_FEN_Msk);

	 	VADC_G0->RCR[1]	   = ((1 << VADC_G_RCR_FEN_Pos) & VADC_G_RCR_FEN_Msk);
	 	VADC_G0->RES[2]	   = ((1 <<  VADC_G_RCR_WFR_Pos) & VADC_G_RCR_WFR_Msk)|
	 					     ((0 <<  VADC_G_RCR_DMM_Pos) & VADC_G_RCR_DMM_Msk);
//FIFO Result accumulation 2
		VADC_G0->RCR[4]	   = ((1 << VADC_G_RCR_FEN_Pos) & VADC_G_RCR_FEN_Msk);
		VADC_G0->RCR[5]	   = ((1 << VADC_G_RCR_FEN_Pos) & VADC_G_RCR_FEN_Msk);
		VADC_G0->RCR[6]	   = ((1 << VADC_G_RCR_FEN_Pos) & VADC_G_RCR_FEN_Msk);

		VADC_G0->RES[7]	   = ((1 <<  VADC_G_RCR_WFR_Pos) & VADC_G_RCR_WFR_Msk)|
				  	  	     ((0 <<  VADC_G_RCR_DMM_Pos) & VADC_G_RCR_DMM_Msk);


//Arbitration

//G0
	 	VADC_G0->ARBCFG    = (3 << VADC_G_ARBCFG_ANONC_Pos & VADC_G_ARBCFG_ANONC_Msk) |
							     // ANONS = 11B: Analog Converter Control: Normal Operation
							     //  The converter is active, conversions are started immediately.
						     (0 << VADC_G_ARBCFG_ARBRND_Pos & VADC_G_ARBCFG_ARBRND_Msk) |
								 // ARBRND = 00B: Arbitration Round Length
								 //  00B: 4 arbitration slots per round (tARB = 4 / fADCD)
						     (0 << VADC_G_ARBCFG_ARBM_Pos & VADC_G_ARBCFG_ARBM_Msk);
								 // ARBM = 0B: Arbitration Mode: The arbiter runs permanently

	 	VADC_G0->ARBPR     = (1 << VADC_G_ARBPR_ASEN0_Pos & VADC_G_ARBPR_ASEN0_Msk);
								//Arbitration Slot 0 Enable
								//Enables the associated arbitration slot of an arbiter round.

//Startup calibration finished?
		while ((VADC_G0->ARBCFG & VADC_G_ARBCFG_CAL_Msk)!=0);


}

/**********************************************************************/
//	Get_Result(uint8_t Group, uint8_t result_number)
//	Read function
// 	This function return the value of the required result register
//  in the indicated group. If the conversion has not been made correctly,
//  the result returned will be the last correct result for this register.
//	If the indicated result register or group number does not exist,
//  the value returned will be 0.
//
/**********************************************************************/
uint32_t Get_Result(uint8_t Group, uint8_t result_number)
{
	if((Group<4)&(result_number<16))
	{

		switch(Group)
			{
			case 0:
				if (VADC_G0->RES[result_number]>>31)//VF indicates a new result
				{
				result10[result_number]=VADC_G0->RES[result_number]&0x00000FFF;
				return VADC_G0->RES[result_number]&0x00000FFF;
				};
				result1=result10[result_number];
				break;
			case 1:
				if (VADC_G1->RES[result_number]>>31)
				{
				result11[result_number]=VADC_G1->RES[result_number]&0x00000FFF;
				return VADC_G1->RES[result_number]&0x00000FFF;
				};
				result1=result11[result_number];
				break;
			case 2:
				if (VADC_G2->RES[result_number]>>31)
				{
				result12[result_number]=VADC_G2->RES[result_number]&0x00000FFF;
				return VADC_G2->RES[result_number]&0x00000FFF;
				};
				result1=result12[result_number];
				break;
			case 3:
				if (VADC_G3->RES[result_number]>>31)
				{
				result13[result_number]=VADC_G3->RES[result_number]&0x00000FFF;
				return VADC_G3->RES[result_number]&0x00000FFF;
				}
				result1=result13[result_number];
				break;

			};
			return result1;
	}
	else
	{
		return 0;
	};
}
